# Zope imports
from persistent.dict import PersistentDict
from zope.annotation.interfaces import IAnnotations
from zope.interface import implements
from zope.formlib import form

# Plone imports
from Products.statusmessages.interfaces import IStatusMessage

# Quills imports
from quills.core import QuillsCoreMessageFactory as _
from quills.core.interfaces import IWeblogConfiguration


ANNO_KEY = 'quills.core.weblogconfiguration'

# XXX These default values should be taken directly from the
# IWeblogViewConfiguration schema.
DEFAULT_VIEW_CONFIG = {
    'only_excerpt_in_weblog_view'   : False,
    'group_by_dates'                : True,
    'entries_per_page'              : 20,
    'topic_images_in_weblog_view'   : True,
    'trackback_enabled'             : False,
    'archive_format'                : '',
    'show_about'                    : True,
    }


class WeblogConfigAnnotations(object):
    """
    >>> from zope.interface.verify import verifyClass
    >>> verifyClass(IWeblogConfiguration, WeblogConfigAnnotations)
    True
    """

    implements(IWeblogConfiguration)

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(self.context)
        self._config = annotations.setdefault(ANNO_KEY, PersistentDict())

    def _get_onlyExcerptInWeblogView(self):
        default = DEFAULT_VIEW_CONFIG['only_excerpt_in_weblog_view']
        return self._config.setdefault('only_excerpt_in_weblog_view', default)
    def _set_onlyExcerptInWeblogView(self, value):
        self._config['only_excerpt_in_weblog_view'] = value
    only_excerpt_in_weblog_view = property(_get_onlyExcerptInWeblogView,
                                           _set_onlyExcerptInWeblogView)

    def _get_groupByDates(self):
        default = DEFAULT_VIEW_CONFIG['group_by_dates']
        return self._config.setdefault('group_by_dates', default)
    def _set_groupByDates(self, value):
        self._config['group_by_dates'] = value
    group_by_dates = property(_get_groupByDates, _set_groupByDates)

    def _get_entriesPerPage(self):
        default = DEFAULT_VIEW_CONFIG['entries_per_page']
        return self._config.setdefault('entries_per_page', default)
    def _set_entriesPerPage(self, value):
        self._config['entries_per_page'] = value
    entries_per_page = property(_get_entriesPerPage, _set_entriesPerPage)

    def _get_showTopicImagesInWeblogView(self):
        default = DEFAULT_VIEW_CONFIG['topic_images_in_weblog_view']
        return self._config.setdefault('topic_images_in_weblog_view', default)
    def _set_showTopicImagesInWeblogView(self, value):
        self._config['topic_images_in_weblog_view'] = value
    topic_images_in_weblog_view = property(_get_showTopicImagesInWeblogView,
                                           _set_showTopicImagesInWeblogView)

    def _get_trackbackEnabled(self):
        default = DEFAULT_VIEW_CONFIG['trackback_enabled']
        return self._config.setdefault('trackback_enabled', default)
    def _set_trackbackEnabled(self, value):
        self._config['trackback_enabled'] = value
    trackback_enabled = property(_get_trackbackEnabled, _set_trackbackEnabled)

    def _get_archiveFormat(self):
        default = DEFAULT_VIEW_CONFIG['archive_format']
        return self._config.setdefault('archive_format', default)
    def _set_archiveFormat(self, value):
        self._config['archive_format'] = value
    archive_format = property(_get_archiveFormat, _set_archiveFormat)
    
    def _get_showAbout(self):
        default = DEFAULT_VIEW_CONFIG['show_about']
        return self._config.setdefault('show_about', default)
    def _set_showAbout(self, value):
        self._config['show_about'] = value
    show_about = property(_get_showAbout, _set_showAbout)


class WeblogConfigEditForm(form.EditForm):
    """Edit form for weblog view configuration.
    """

    form_fields = form.Fields(IWeblogConfiguration)
    label = u'Weblog View Configuration'

    def setUpWidgets(self, ignore_request=False):
        self.adapters = {}
        wvconfig = IWeblogConfiguration(self.context)
        self.widgets = form.setUpEditWidgets(
            self.form_fields, self.prefix, wvconfig, self.request,
            adapters=self.adapters, ignore_request=ignore_request
            )

    @form.action("submit")
    def submit(self, action, data):
        """
        """
        wvconfig = IWeblogConfiguration(self.context)
        form.applyChanges(wvconfig, self.form_fields, data)
        msg = _(u'Configuration saved.')
        IStatusMessage(self.request).addStatusMessage(msg, type='info')



# Zope3 imports
from zope.interface import Interface
from zope import schema


# Quills import
from basecontent import IBaseContent
from topic import ITopicContainer
from topic import IAuthorContainer
from archive import IWeblogArchive

from quills.core import QuillsCoreMessageFactory as _

class IReadWeblog(IWeblogArchive,
                  ITopicContainer,
                  IAuthorContainer,
                  IBaseContent):
    """
    """

    def getId():
        """Return the id of the weblog.
        """

    def getTitle():
        """Return the title of the weblog.
        """

    def hasEntry(id):
        """Return True if this IReadWeblog holds an entry with an id of `id',
        and False otherwise.
        """

    def getEntry(id):
        """Return the IWeblogEntry associated with `id', or None if there is no
        such entry.
        """

    def getEntries(maximum=None, offset=0):
        """Return a sequence of published IWeblogEntry instances, sorted by
        publishing date.  Only return a maximum of `maximum' (where None means
        no limit), and use `offset' to determine where in the full sequence the
        returned sequence starts from.
        """


class IWriteWeblog(Interface):
    """
    """

    def addEntry(title, excerpt, text, topics=[], id=None, pubdate=None, mimetype=None):
        """Add an entry from the provided arguments.  If id is None, normalize
        the title to create an id.  If pubdate is None, ignore it.
        If mimetype is None, use the default.
        Return the new entry.
        """

    def addFile(content, mimetype, id=None, title=''):
        """Add a file-ish object corresponding to `mimetype' that stores
        `content'.  If `id' is provided, use it for the file; otherwise generate
        a new (unique) id.  If `id' is not provided, try normalizing a provided
        `title' argument (if available) to create the id.
        Return the added object.
        """


class IEditWeblog(Interface):
    """
    """

    def getDrafts(maximum=None, offset=0):
        """Return a sequence of unpublished IWeblogEntry instances, sorted by
        creation date.  Only return a maximum of `maximum' (where None means no
        limit), and use `offset' to determine where in the full sequence the
        returned sequence starts from.
        """

    def getAllEntries(maximum=None, offset=0):
        """Return a sequence all IWeblogEntry instances (published or not),
        sorted by publishing date.  Only return a maximum of `maximum' (where
        None means no limit), and use `offset' to offset where in the full
        sequence the returned sequence starts from.
        """

    def deleteEntry(entry_id):
        """Delete the IWeblogEntry corresponding to `entry_id'.
        """


class IWeblog(IReadWeblog, IWriteWeblog, IEditWeblog):
    """A weblog.
    """

    def getWeblogContentObject():
        """Return the content object that can be adapted to IWeblog, and which
        this IWeblog represents.
        """


class IWeblogConfiguration(Interface):
    """
    """

    only_excerpt_in_weblog_view = schema.Bool(
        title=_(u'label_only_excerpt_in_weblog_view', default=u'Only excerpt in weblog view.'),
        description=_(u'help_only_excerpt_in_weblog_view', default=u'When enabled, show only the title and excerpt in the main weblog view.  If an entry has no excerpt, only its title will be displayed.'),
        default=False,
        )

    group_by_dates = schema.Bool(
        title=_(u'label_group_by_dates', default=u'Group by dates'),
        description=_(u'help_group_by_dates', default=u'When enabled, entries will be grouped under a header showing the date. Otherwise, the entries will be just be shown underneath eachother.'),
        default=True,
        )

    entries_per_page = schema.Int(
        title=_(u'label_entries_per_page', default=u'Entries Per Page'),
        description=_(u'help_entries_per_page', default=u'Select the number of weblog entries you would like to display on the front page and any other batched pages.'),
        default=20,
        )

    topic_images_in_weblog_view = schema.Bool(
        title=_(u'label_show_topic_images_in_weblog_view', default=u'Show Topic Images In Weblog View?'),
        description=_(u'help_show_topic_images_in_weblog_view', default=u'This controls the display of topic images in the weblog view.'),
        default=True,
        )

    trackback_enabled = schema.Bool(
        title=_(u'label_trackback_enabled', default=u'Enable the receiving of trackback pings?'),
        description=_(u'help_trackback_enabled', default=u'This controls whether trackback are enabled in the weblog'),
        default=False,
        )

    archive_format = schema.TextLine(
        title=_(u'label_archive_url_prefix', default=u'Archive URL prefix'),
        description=_(u'help_archive_url_prefix', default="Allows for (optionally) injecting a segment into archive URLs after the weblog segment. E.g. to have URLs like [weblog]/archive/2007/07/21/entry_id, enter 'archive' here."),
        default=u'',
        required=False,
        )
    
    show_about = schema.Bool(
        title=_(u"Show 'About' info"),
        description=_(u"If selected, the item creator and modification date will be shown."),
        default=True,
        )


class IWeblogLocator(Interface):
    """Find a weblog."""

    def find():
        """Some IWeblog implementing object or empty list is returned.
        """



# Zope3 imports
from zope.interface import Interface

# Local imports
from basecontent import IBaseContent


class IReadWeblogEntry(IBaseContent):
    """A weblog entry.
    """

    def getId():
        """Return the id of the entry.
        """

    def getTitle():
        """Return the title of the entry.
        """

    def getTopics():
        """Return a sequence of ITopic instances for this IWeblogEntry.
        """

    def getAuthors():
        """Return a sequence of IAuthorTopic instances for this IWeblogEntry.
        This will usually be a sequence of length == 1.
        """

    def getExcerpt():
        """Return an excerpt/description of the post.
        """

    def getText():
        """Return the HTML text body of the post.
        """

    def getPublicationDate():
        """Return a DateTime instance for when this IWeblogEntry was/will-be
        published.
        """

    def getWeblogEntryContentObject():
        """Return the content object that can be adapted to IWeblogEntry, and
        which this IWeblogEntry represents.
        """

    def getMimeType():
        """return the mimetype of the entry (i.e. text/plain, text/html etc.)"""
        pass


class IEditWeblogEntry(Interface):
    """
    """

    def setTitle(title):
        """
        """

    def setTopics(topic_ids):
        """
        """

    def setExcerpt(excerpt):
        """
        """

    def setText(text, mimetype=None):
        """
        If mimetype is None, use the default.
        """

    def edit(title, excerpt, text, topics, mimetype=None):
        """
        If mimetype is None, use the default.
        """

    def setPublicationDate(datetime):
        """Set when this IWeblogEntry was/will-be published.
        """


class IWeblogEntry(IReadWeblogEntry, IEditWeblogEntry):
    """A weblog entry.
    """


class IWorkflowedWeblogEntry(IWeblogEntry):
    """A very simple workflow implementation for weblog entries.
    """

    def publish(pubdate=None):
        """Publish this weblog entry.  Do nothing if it is already published.
        `pubdate' defaults to datetime.now().
        """

    def retract():
        """Retract this weblog entry to 'draft' status.  Do nothing if it is
        already a draft.
        """

    def isPublished():
        """Return True if this weblog entry is currently published, False
        otherwise.
        """



# Zope imports
from zope.interface import Interface
from zope import schema
from zope.i18nmessageid import MessageFactory

# Quills imports
from quills.core.interfaces import IWeblogConfiguration


class IPossibleWeblog(Interface):
    """A marker interface for representing what *could* be a weblog.
    """


class IWeblogEnhanced(Interface):
    """A marker interface to indicate an item that has weblog functionality.
    """


class IPossibleWeblogEntry(Interface):
    """A marker interface for representing what *could* be a weblog entry.
    """



# Zope3 imports
from zope.interface import Interface

from quills.core.interfaces.basecontent import IBaseContent


class IBaseView(IBaseContent):
    """Base Quills view.
    """

    def isWeblogContent(obj=None):
        """Return True if obj is one of the weblog content types in the sense of
        the interface it provides; False otherwise.
        If `obj' is None, then self.context is used.
        """

    def getArchiveURLFor(weblogentry):
        """Return an archive URL for the object represented by the IWeblogEntry.
        """

    def getArchivePathFor(weblogentry):
        """Return an archive path for the object represented by the
        IWeblogEntry.
        """

    def displayingOneEntry(context, weblogentry):
        """Return True if the absolute_url of context equals that for
        weblogentry; False otherwise.
        """

    def isDiscussionAllowedFor(obj):
        """Check if discussion is allowed on obj, but in a way that can handle
        obj being a catalog brain.
        """

    def getCommentCountFor(obj):
        """Check how many discussion comments there are on obj, but in a way
        that can handle obj being a catalog brain.
        """


class IWeblogView(IBaseView):
    """An interface for a helper view.
    """

    def getConfig():
        """Return an IWeblogViewConfiguration instance for the weblog.
        """

    def getWeblogEntriesDates(entries_dict):
        """Return a sorted list of dates for dict of entries.
        """

    def sortWeblogEntriesToDates(lazy_entries, resolution='day'):
        """Sort the lazy entries into a dictionary keyed on dates with values a
        lists of WeblogEntry (catalog brains) that were posted on those days.
        The default 'resolution' is 'day', meaning the returned dictionary
        will be keyed on individual days.  Other options are 'month' and 'year'.
        """


class IWeblogEntryView(IBaseView):
    """
    """

    def getWeblogEntry():
        """
        """


class ITopicView(IWeblogView):
    """
    """

    def getLastModified():
        """Return the datetime for when the most recently modified IWeblogEntry
        in this topic was modified.
        """



# Local imports
from basecontent import IBaseContent


class ITopic(IBaseContent):
    """A keyword, with an optionally associated image, that knows how to find
    content tagged with it.
    """

    def getId():
        """Return the keyword/id for this topic.
        """

    def getKeywords():
        """Return a sequence of keywords that this topic represents.
        """

    def getTitle():
        """Return title.
        """

    def getDescription():
        """Return the description for this topic.
        """

    def getImage():
        """Return an image object for this topic.
        """

    def getEntries(maximum=None, offset=0):
        """Return a sequence of published IWeblogEntry instances, sorted by
        publishing date.  Only return a maximum of `maximum' (where None means no
        limit), and use `offset' to determine where in the full sequence the
        returned sequence starts from.
        """

    def __len__():
        """Return the number of posts for this ITopic.
        """

    def __str__():
        """Return a string representation of one or more the keywords.
        """


class IAuthorTopic(ITopic):
    """Represents a topic that queries based on authors (aka 'Creators').
    """


class ITopicContainer(IBaseContent):
    """Represents a container for ITopic objects.
    """

    def getTopics():
        """Return a sequence of ITopic instances for this blog.
        """

    def getTopicById(id):
        """Return an ITopic instance corresponding to `id'.
        """


class IAuthorContainer(IBaseContent):
    """Represents a container for IAuthor objects.
    """

    def getAuthors():
        """Return a sequence of IAuthorTopic instances, one for every portal
        member id that has posted to this IWeblog.
        """

    def getAuthorById(id):
        """Return an IAuthorTopic instance for id.
        """



from basecontent import IBaseContent
from weblog import IReadWeblog
from weblog import IWeblog
from weblog import IEditWeblog
from weblog import IWriteWeblog
from weblog import IWeblogConfiguration
from weblog import IWeblogLocator
from weblogentry import IReadWeblogEntry
from weblogentry import IEditWeblogEntry
from weblogentry import IWeblogEntry
from weblogentry import IWorkflowedWeblogEntry
from archive import IWeblogArchive
from archive import IWeblogArchiveContainer
from topic import IAuthorTopic
from topic import ITopic
from topic import ITopicContainer
from topic import IAuthorContainer
from enabled import IPossibleWeblog
from enabled import IPossibleWeblogEntry
from enabled import IWeblogEnhanced

# PYFLAKES
IBaseContent, IReadWeblog, IWeblog, IEditWeblog, IWriteWeblog
IWeblogConfiguration, IReadWeblogEntry, IEditWeblogEntry, IWeblogEntry
IWorkflowedWeblogEntry, IWeblogArchive, IWeblogArchiveContainer, IAuthorTopic
ITopic, IPossibleWeblog, IPossibleWeblogEntry, IWeblogEnhanced, ITopicContainer
IAuthorContainer


# Zope imports
from zope.interface import Interface


class IBaseContent(Interface):
    """Base interface for Quills content-ish types.
    """

    def getWeblog():
        """Return the IWeblog object that this content lives in.  This may be an
        object adapted to IWeblog.
        """

    def getWeblogContentObject():
        """Return the content object that can be adapted to IWeblog, and in
        which this content lives.
        """



# Local imports
from basecontent import IBaseContent

class IWeblogArchive(IBaseContent):
    """An archive folder for a weblog.
    """

    def getSubArchives():
        """Get a listing of all sub-archives.
        """

    def getEntries(maximum=None, offset=0):
        """Return a sequence of published IWeblogEntry instances, sorted by
        publishing date.  Only return a maximum of `maximum' (where None means no
        limit), and use `offset' to determine where in the full sequence the
        returned sequence starts from.
        """

    def __len__():
        """Return the number of IWeblogEntry instances in this archive.
        """


class IWeblogArchiveContainer(IWeblogArchive):
    """An interface for the root archive container for an IWeblog when there is
    an extra 'archive' URL segment in use: e.g. '.../weblog/archive/2007/07'
    """
